# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pouty', 'pouty.stdlib']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'pouty',
    'version': '1.0.0',
    'description': 'C++-style IO in Python!',
    'long_description': '# Pouty: a very silly library bringing weird C++ features to Python since 2022\n\nDo you like Python? Wanna use std::cout in it? No? I didn\'t ask!\n\n`pouty` is a Python package that will make you pout with it\'s ability to use C++ style IO in Python.\n\n## FAQ:\n\n- **Is it fast?**\n  No.\n- **Does C++ I/O even make sense in this context?**\n  That\'s for you to judge.\n- **Is it faster than Python\'s print()?**\n  While it may *occasionally* avoid a string creation or contcatenation, it does so through multiple operator overloads and several function calls + type checks. Therefore, no.\n- **Why is C++\'s iostream fast?**\n  C++ is a compiled language with static typing and function call inlining.\n- **Why make this?** \n  Because I can.\n\n\n## Example code (`examples/test.py`):\n\n```python\nfrom pouty.iostream import *;\nimport random;\n\nstd.cout << "Hello, world!" << std.endl\n\nclass CustomOutput: #custom operator<< and operator>> overload supported\n    is_pouty_stream = True\n    def __init__(self) -> None:\n        self.x = ref(0)\n        pass\n\n    def __lshift__(self, other: any) -> None:\n        other << self.x[...]\n    \n    def __rshift__(self, other: any) -> None:\n        other >> self.x\n\n# Python [unfortunately, perhaps fortunately!] does not have implicit reference parameters.\nx = ref(0)\nstd.cin >> x\nstd.cout << x << std.endl # uses the __str__ method of ref\nstd.cout << x[...] << std.endl # dereferences the ref and prints the int\n\nx = CustomOutput()\nstd.cin >> x\nstd.cout << x << std.endl\n\nx = random.random() #precision is supported too!\nstd.cout << x << std.endl\nstd.cout << std.precision(3) \nstd.cout << x << std.endl\n\n\nusing_namespace(globals(), "std")\ncout << "Hello, world!" << endl # bringing namespace pollution into Python since 2022 :)\n\nfrom pouty.stdlib.h import *;\n\nprintf("%s", "Hello, world!")\nprintf("%d", 123) #note that precision is not supported yet here (feel free to contrubute it)\nputs("Hello, world!")\n```\n\n## Installation\n\n`pip install pouty`\n\n## Contributing\n\nPull requests are welcome. For major changes, please open an issue first to discuss the change.\n',
    'author': 'Om Duggineni',
    'author_email': 'omduggineni@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import sys
import ctypes
from math import *


class ref: #because python doesn't support pass-by-reference
    def __init__(self, value):
        self.value = value
    
    def __getitem__(self, index: Ellipsis):
        if index == ...:
            return self.value
    def __setitem__(self, index: Ellipsis, value: any):
        if index == ...:
            self.value = value

    def __str__(self):
        return str(self.value)

def _cround(number: float, precision: int):
    #round to sig figs
    return round(number, -int(floor(log10(abs(number))))+precision-1)

class modifier_base:
    ...

class cout_base:
    def __init__(self, stream=sys.stdout):
        self.stream = stream
        self.float_precision = 6
    #left shift operator
    def __lshift__(self, other):
        if isinstance(other, modifier_base):
            other(self)
            return self
        elif isinstance(other, float):
            self.stream.write(str(_cround(other, self.float_precision)))
            return self
        else:
            #check if lshift operator is overloaded
            if "is_pouty_stream" in other.__class__.__dict__ and other.__class__.is_pouty_stream:
                other.__lshift__(self)
            else:
                sys.stdout.write(str(other))
            return self

class cin_base:
    def __init__(self, stream=sys.stdin):
        self.stream = stream
    #right shift operator
    def __rshift__(self, other: ref):
        if "is_pouty_stream" in other.__class__.__dict__ and other.__class__.is_pouty_stream:
            other.__rshift__(self)
            return self
        if isinstance(other[...], str):
            other[...] = sys.stdin.readline()
        elif isinstance(other[...], float):
            other[...] = float(sys.stdin.readline())
        elif isinstance(other[...], int):
            other[...] = int(sys.stdin.readline())
        elif hasattr(other[...], "__rshift__"):
            other[...] = other[...].__rshift__(self)
        else:
            raise RuntimeError("Not implemented yet")
        return self

def modifier(function: callable): #e.g. std.endl
    class _modifier(modifier_base):
        def __call__(self, streamer):
            return function(streamer)
    return _modifier()

def nested_modifier(function: callable): #e.g. std.precision()
    def modifier_generator(*args):
        class _modifier(modifier_base):
            def __call__(self, streamer: cout_base):
                return function(streamer, *args)
        return _modifier()
    return modifier_generator
            
class std:
    cout = cout_base(sys.stdout)
    cerr = cout_base(sys.stderr)
    clog = cerr
    cin = cin_base(sys.stdin)

    @modifier
    def endl(streamer):
        streamer.stream.write("\n")
        streamer.stream.flush()
    
    @nested_modifier
    def precision(streamer, precision: int):
        streamer.float_precision = precision

def using_namespace(globals_ref, namespace):
    if namespace == "std":
        for key, value in std.__dict__.items():
            if key[:2] != "__" and key[-2:] != "__":
                globals_ref[key] = value